local hcTk = HCToolkit

function hcTk.InitDB()
    hcTk.LSM = LibStub:GetLibrary("LibSharedMedia-3.0")
    hcTk.LSM:Register("font", "Myriad Condensed Web", "Interface\\Addons\\HCToolkit\\Media\\MyriadCondensedWeb.ttf")
    hcTk.LSM:Register("font", "Roboto-Condensed", "Interface\\Addons\\HCToolkit\\Media\\RobotoCondensed.ttf")
    hcTk.LSM:Register("font", "Roboto-Mono Medium", "Interface\\Addons\\HCToolkit\\Media\\RobotoMonoMedium.ttf")
    hcTk.LSM:Register("sound", "ThatsWrong", "Interface\\Addons\\HCToolkit\\Media\\ThatsWrong.mp3")

    local defaults = {
        profile = {
            Assignments = {
                TOS = {
                    MaidenSoakBlacklist = {
                        [1] = "", [2] = "", [3] = "", [4] = "", [5] = "",
                    },
                    AvatarMatrixSoaks = {
                        Left = { [1] = "", [2] = "", [3] = "", [4] = "", [5] = "", [6] = "", },
                        Right = { [1] = "", [2] = "", [3] = "", [4] = "", [5] = "", [6] = "", },
                    },
                },
            },
            DebugMode = false,
            Frames = {
                Group = {
                    Anchor = "LEFT",
                    Text = {
                        Align = "LEFT",
                        Flag = "OUTLINE",
                        Font = "Roboto-Mono Medium",
                        Size = 14,
                    },
                    X = 0,
                    Y = 0,
                },
                Personal = {
                    Anchor = "TOP",
                    Text = {
                        Align = "CENTER",
                        Flag = "OUTLINE",
                        Font = "Roboto-Mono Medium",
                        Size = 32,
                    },
                    X = 0,
                    Y = 0,
                },
            },
            Encounters = {
                TOS = {
                    Maiden = {
                        SoakPersonal = true,
                        SoakGroup = false,
                    },
                    Avatar = {
                        DarkMarksGroup = true,
                        DarkMarkPersonal = true,
                        TaintedSoakGroup = "CURRENT",
                        TaintedSoakPersonal = true,
                    }
                },
            },
            MattySmite = true,
        },
    }

    local options = {
        name = "HCToolkit",
        handler = HCToolkit,
        type = "group",
        args = {
            core = {
                name = "Core Features",
                type = "group",
                args = {
                    DebugMode = {
                        order = 1,
                        type = "toggle",
                        name = "Debug Mode",
                        desc = "Toggles Debug Mode",
                        get = function() return hcTk.GenericGet("DebugMode") end,
                        set = function(_, v) hcTk.GenericSet("DebugMode", v) end,
                    },
                    MattySmite = {
                        order = 2,
                        type = "toggle",
                        name = "Matty Smite",
                        desc = "Enabling This Allows Matty Smite To Help Lead You To Victory",
                        get = function() return hcTk.GenericGet("MattySmite") end,
                        set = function(_, v) hcTk.GenericSet("MattySmite", v) end,
                    },
                },
            },
            groupFrame = {
                name = "Group Frame",
                type = "group",
                args = {
                    GroupUnlocked = {
                        order = 1,
                        type = "toggle",
                        name = "Unlocked",
                        desc = "Toggles Unlock State Of Group Frame",
                        get = function() return HCToolkitGroupFrame.Unlocked end,
                        set = function(_, v) hcTk.FrameToggleUnlock(HCToolkitGroupFrame, v) end,
                    },
                    GroupFont = {
                        order = 2,
                        type = "select",
                        dialogControl = "LSM30_Font",
                        name = "Group Font",
                        desc = "Default: Roboto Mono Medium",
                        values = HCToolkit.LSM:HashTable("font"),
                        get = function() return hcTk.DB.profile.Frames.Group.Text.Font end,
                        set = function(_, v) hcTk.FrameSetFontProperty(HCToolkitGroupFrame, hcTk.DB.profile.Frames.Group.Text, "Font", v) end,
                    },
                    GroupFontFlag = {
                        order = 3,
                        type = "select",
                        name = "Group Font Flag",
                        desc = "Default: OUTLINE",
                        values = {["OUTLINE"] = "OUTLINE", ["THICKOUTLINE"] = "THICKOUTLINE", ["MONOCHROME"] = "MONOCHROME", ["NONE"] = "NONE"},
                        get = function() return hcTk.DB.profile.Frames.Group.Text.Flag end,
                        set = function(_, v) hcTk.FrameSetFontProperty(HCToolkitGroupFrame, hcTk.DB.profile.Frames.Group.Text, "Flag", v) end,
                    },
                    GroupFontAlign = {
                        order = 4,
                        type = "select",
                        name = "Group Align",
                        desc = "Default: LEFT",
                        values = {["LEFT"] = "LEFT", ["CENTER"] = "CENTER", ["RIGHT"] = "RIGHT"},
                        get = function() return hcTk.DB.profile.Frames.Group.Text.Align end,
                        set = function(_, v) hcTk.FrameSetFontProperty(HCToolkitGroupFrame, hcTk.DB.profile.Frames.Group.Text, "Align", v) end,
                    },
                    GroupFontSize = {
                        order = 5,
                        type = "range",
                        name = "Group Font Size",
                        desc = "Default: 14",
                        min = 8,
                        max = 72,
                        step = 1,
                        get = function() return hcTk.DB.profile.Frames.Group.Text.Size end,
                        set = function(_, v) hcTk.FrameSetFontProperty(HCToolkitGroupFrame, hcTk.DB.profile.Frames.Group.Text, "Size", v) end,
                    },
                    GroupDemoMode = {
                        order = 6,
                        type = "execute",
                        name = "Group Demo Mode",
                        desc = "Shows Group Frame For Demo / Setup Purposes",
                        func = function() hcTk.FrameDemoMode(HCToolkitGroupFrame, not HCToolkitGroupFrame.DemoMode) end,
                    },
                }
            },
            personalFrame = {
                name = "Personal Frame",
                type = "group",
                args = {
                    PersonalUnlocked = {
                        order = 1,
                        type = "toggle",
                        name = "Unlocked",
                        desc = "Toggles Unlock State Of Personal Frame",
                        get = function() return HCToolkitPersonalFrame.Unlocked end,
                        set = function(_, v) hcTk.FrameToggleUnlock(HCToolkitPersonalFrame, v) end,
                    },
                    PersonalFont = {
                        order = 2,
                        type = "select",
                        dialogControl = "LSM30_Font",
                        name = "Personal Font",
                        desc = "Default: Roboto Mono Medium",
                        values = HCToolkit.LSM:HashTable("font"),
                        get = function() return hcTk.DB.profile.Frames.Personal.Text.Font end,
                        set = function(_, v) hcTk.FrameSetFontProperty(HCToolkitPersonalFrame, hcTk.DB.profile.Frames.Personal.Text, "Font", v) end,
                    },
                    PersonalFontFlag = {
                        order = 3,
                        type = "select",
                        name = "Personal Font Flag",
                        desc = "Default: OUTLINE",
                        values = {["OUTLINE"] = "OUTLINE", ["THICKOUTLINE"] = "THICKOUTLINE", ["MONOCHROME"] = "MONOCHROME", ["NONE"] = "NONE"},
                        get = function() return hcTk.DB.profile.Frames.Personal.Text.Flag end,
                        set = function(_, v) hcTk.FrameSetFontProperty(HCToolkitPersonalFrame, hcTk.DB.profile.Frames.Personal.Text, "Flag", v) end,
                    },
                    PersonalFontAlign = {
                        order = 4,
                        type = "select",
                        name = "Personal Align",
                        desc = "Default: CENTER",
                        values = {["LEFT"] = "LEFT", ["CENTER"] = "CENTER", ["RIGHT"] = "RIGHT"},
                        get = function() return hcTk.DB.profile.Frames.Personal.Text.Align end,
                        set = function(_, v) hcTk.FrameSetFontProperty(HCToolkitGroupFrame, hcTk.DB.profile.Frames.Personal.Text, "Align", v) end,
                    },
                    PersonalFontSize = {
                        order = 5,
                        type = "range",
                        name = "Personal Font Size",
                        desc = "Default: 32",
                        min = 8,
                        max = 72,
                        step = 1,
                        get = function() return hcTk.DB.profile.Frames.Personal.Text.Size end,
                        set = function(_, v) hcTk.FrameSetFontProperty(HCToolkitPersonalFrame, hcTk.DB.profile.Frames.Personal.Text, "Size", v) end,
                    },
                    PersonalDemoMode = {
                        order = 6,
                        type = "execute",
                        name = "Personal Demo Mode",
                        desc = "Shows Personal Frame For Demo / Setup Purposes",
                        func = function() hcTk.FrameDemoMode(HCToolkitPersonalFrame, not HCToolkitPersonalFrame.DemoMode) end,
                    },
                }
            },
            encountersTOS = {
                name = "Tomb Of Sargeras",
                type = "group",
                args = {
                    Maiden = {
                        order = 1,
                        type = "header",
                        name = "Maiden Of Vigilance"
                    },
                    MaidenPersonalSoak = {
                        order = 2,
                        type = "toggle",
                        name = "Personal Soak Assignment",
                        desc = "Shows your soak assignment on the personal frame",
                        get = function() return hcTk.GenericGetBossOption("TOS", "Maiden", "SoakPersonal") end,
                        set = function(_, v) hcTk.GenericSetBossOption("TOS", "Maiden", "SoakPersonal", v) end,
                    },
                    MaidenGroupSoak = {
                        order = 3,
                        type = "toggle",
                        name = "Group Soak Assignment",
                        desc = "Shows the groups soak assignments on the group frame",
                        get = function() return hcTk.GenericGetBossOption("TOS", "Maiden", "SoakGroup") end,
                        set = function(_, v) hcTk.GenericSetBossOption("TOS", "Maiden", "SoakGroup", v) end,
                    },
                    Avatar = {
                        order = 4,
                        type = "header",
                        name = "Avatar Of Sargeras",
                    },
                    AvatarGroupDarkMarks = {
                        order = 5,
                        type = "toggle",
                        name = "Group Dark Marks",
                        desc = "Shows the dark marks, and their timers in the group frame",
                        get = function() return hcTk.GenericGetBossOption("TOS", "Avatar", "DarkMarksGroup") end,
                        set = function(_, v) hcTk.GenericSetBossOption("TOS", "Avatar", "DarkMarksGroup", v) end,
                    },
                    AvatarPersonalDarkMark = {
                        order = 6,
                        type = "toggle",
                        name = "Personal Dark Mark",
                        desc = "Shows your dark mark, its timer, and position in the personal frame",
                        get = function() return hcTk.GenericGetBossOption("TOS", "Avatar", "DarkMarkPersonal") end,
                        set = function(_, v) hcTk.GenericSetBossOption("TOS", "Avatar", "DarkMarkPersonal", v) end,
                    },
                    AvatarGroupTaintedSoak = {
                        order = 7,
                        type = "select",
                        name = "Group Tainted Soak",
                        desc = "Shows the groups for tainted soak, by default it shows only the group you are in, but can show both",
                        values = {["CURRENT"] = "Current Group", ["BOTH"] = "Both Groups", ["NONE"] = "None Shown"},
                        get = function() return hcTk.GenericGetBossOption("TOS", "Avatar", "TaintedSoakGroup") end,
                        set = function(_, v) hcTk.GenericSetBossOption("TOS", "Avatar", "TaintedSoakGroup", v) end,
                    },
                    AvatarPersonalTaintedSoak = {
                        order = 8,
                        type = "toggle",
                        name = "Personal Tainted Soak",
                        desc = "Shows when you are up next to soak Tainted Essence",
                        get = function() return hcTk.GenericGetBossOption("TOS", "Avatar", "TaintedSoakPersonal") end,
                        set = function(_, v) hcTk.GenericSetBossOption("TOS", "Avatar", "TaintedSoakPersonal", v) end,
                    },
                },
            },
        },
    }

    hcTk.DB = LibStub("AceDB-3.0"):New("HCToolkitDB", defaults, true);
    LibStub("AceConfig-3.0"):RegisterOptionsTable("HCToolkit", options);
    hcTk.OptionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("HCToolkit", "HCToolkit");
end